@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018 Shinichi Takii, shinichi.takii@gmail.com
+# Copyright (C) 2018 Shinichi Takii, shinichi.takii@shaketh.com
 #
 # This module is part of python-ddlparse and is released under
 # the BSD License: https://opensource.org/licenses/BSD-3-Clause
@@ -8,10 +8,10 @@
 from .ddlparse import *
 
 __copyright__    = 'Copyright (C) 2018-2020 Shinichi Takii'
-__version__      = '1.6.1'
+__version__      = '1.9.0'
 __license__      = 'BSD-3-Clause'
 __author__       = 'Shinichi Takii'
-__author_email__ = 'shinichi.takii@gmail.com'
+__author_email__ = 'shinichi.takii@shaketh.com'
 __url__          = 'http://github.com/shinichi-takii/ddlparse'
 
 __all__ = ['DdlParse', 'DdlParseTable', 'DdlParseColumn', 'DdlParseColumnDict']
