@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2018 Shinichi Takii, shinichi.takii@gmail.com
+# Copyright (C) 2018 Shinichi Takii, shinichi.takii@shaketh.com
 #
 # This module is part of python-ddlparse and is released under
 # the BSD License: https://opensource.org/licenses/BSD-3-Clause
@@ -139,17 +139,78 @@ class DdlParseColumn(DdlParseTableColumnBase):
 
     @constraint.setter
     def constraint(self, constraint):
-        self._constraint = None if constraint is None else constraint.upper()
 
-        self._not_null = False if self._constraint is None or not re.search(r"(NOT NULL|PRIMARY KEY)", self._constraint) else True
-        self._pk = False if self._constraint is None or not re.search("PRIMARY KEY", self._constraint) else True
-        self._unique = False if self._constraint is None or not re.search("UNIQUE", self._constraint) else True
+        # Compatibility v1.6.1 and earlier
+        if type(constraint) is str:
+            self._constraint = None if constraint is None else constraint.upper()
+
+            self._not_null = False if self._constraint is None or not re.search(r"(NOT NULL|PRIMARY KEY)", self._constraint) else True
+            self._pk = False if self._constraint is None or not re.search("PRIMARY KEY", self._constraint) else True
+            self._unique = False if self._constraint is None or not re.search("UNIQUE", self._constraint) else True
+
+            self._comment = None
+            if constraint is not None:
+                matches = re.findall(r"(?:\bCOMMENT\b\s+)(['\"])(.+)\1", constraint, re.IGNORECASE)
+                if len(matches) > 0:
+                    self._comment = matches[0][1]
+
+            return
+
+
+        # v1.7.0 or later
+
+        self._constraint = None if constraint is None else ' '.join(constraint).upper()
+
+        constraints = {}
+        constraints['null'] = ''
+        constraints['auto_increment'] = ''
+        constraints['key'] = ''
+        constraints['default'] = ''
+        constraints['comment'] = ''
+        constraints['encode'] = ''
+        constraints['distkey'] = ''
+        constraints['sortkey'] = ''
+        constraints['character_set'] = ''
+
+        if constraint:
+            for constraint_name, val in constraint.items():
+                constraints[constraint_name] = val
+
+
+        self._pk = True if re.search("PRIMARY", constraints['key'], re.IGNORECASE) else False
+
+        self._not_null = False
+        if self._pk or re.search(r"(NOT\s+NULL)", constraints['null'], re.IGNORECASE):
+            self._not_null = True
+
+        self._unique = True if re.search("UNIQUE", constraints['key'], re.IGNORECASE) else False
+
+        self._auto_increment = True if len(constraints['auto_increment']) > 0 else False
+        self._distkey        = True if len(constraints['distkey']) > 0 else False
+        self._sortkey        = True if len(constraints['sortkey']) > 0 else False
+        self._character_set  = constraints['character_set'] if constraints['character_set'] else None
+
+        self._encode = None
+        if constraint is not None:
+            matches = re.findall(r"\bENCODE\s+([A-Za-z0-9]+)\b", constraints['encode'], re.IGNORECASE)
+            if len(matches) > 0:
+                self._encode = matches[0]
+
+        self._default = None
+        if constraint is not None:
+            matches = re.findall(
+                r"\bDEFAULT\b\s+(?:((?:[A-Za-z0-9_\.\'\" -\{\}]|[^\x01-\x7E])*\:\:(?:character varying)?[A-Za-z0-9\[\]]+)|(?:\')((?:\\\'|[^\']|,)+)(?:\')|(?:\")((?:\\\"|[^\"]|,)+)(?:\")|([^,\s]+))",
+                constraints['default'],
+                re.IGNORECASE)
+            if len(matches) > 0:
+                self._default = ''.join(matches[0])
 
         self._comment = None
         if constraint is not None:
-            matches = re.findall(r"(?:\bCOMMENT\b\s+)(['\"])(.+)\1", constraint, re.IGNORECASE)
+            matches = re.findall(r"\bCOMMENT\b\s+(?:(?:\')((?:\\\'|[^\']|,)+)(?:\')|(?:\")((?:\\\"|[^\"]|,)+)(?:\")|([^,\s]+))", constraints['comment'], re.IGNORECASE)
             if len(matches) > 0:
-                self._comment = matches[0][1]
+                self._comment = ''.join(matches[0])
+
 
     @property
     def comment(self):
@@ -196,13 +257,37 @@ class DdlParseColumn(DdlParseTableColumnBase):
     def unique(self, flag):
         self._unique = flag
 
+    @property
+    def auto_increment(self):
+        return self._auto_increment
+
+    @property
+    def distkey(self):
+        return self._distkey
+
+    @property
+    def sortkey(self):
+        return self._sortkey
+
+    @property
+    def encode(self):
+        return self._encode
+
+    @property
+    def default(self):
+        return self._default
+
+    @property
+    def character_set(self):
+        return self._character_set
+
     @property
     def bigquery_data_type(self):
         """Get BigQuery Legacy SQL data type"""
 
         # BigQuery data type = {source_database: [data type, ...], ...}
         BQ_DATA_TYPE_DIC = OrderedDict()
-        BQ_DATA_TYPE_DIC["STRING"] = {None: [re.compile(r"(CHAR|TEXT|CLOB|JSON|UUID)")]}
+        BQ_DATA_TYPE_DIC["STRING"] = {None: [re.compile(r"(STRING|CHAR|TEXT|CLOB|JSON|UUID)")]}
         BQ_DATA_TYPE_DIC["INTEGER"] = {None: [re.compile(r"INT|SERIAL|YEAR")]}
         BQ_DATA_TYPE_DIC["FLOAT"] = {None: [re.compile(r"(FLOAT|DOUBLE)"), "REAL", "MONEY"]}
         BQ_DATA_TYPE_DIC["DATETIME"] = {
@@ -213,6 +298,7 @@ class DdlParseColumn(DdlParseTableColumnBase):
         BQ_DATA_TYPE_DIC["DATE"] = {None: ["DATE"]}
         BQ_DATA_TYPE_DIC["TIME"] = {None: ["TIME"]}
         BQ_DATA_TYPE_DIC["BOOLEAN"] = {None: [re.compile(r"BOOL")]}
+        BQ_DATA_TYPE_DIC["BYTES"] = {None: ["BYTES", "BINARY", "VARBINARY", "BYTEA"]}
 
         for bq_type, conditions in BQ_DATA_TYPE_DIC.items():
             for source_db, source_datatypes in conditions.items():
@@ -496,49 +582,20 @@ class DdlParse(DdlParseBase):
         map(CaselessKeyword, "CREATE, TABLE, TEMP, CONSTRAINT, NOT NULL, PRIMARY KEY, UNIQUE, UNIQUE KEY, FOREIGN KEY, REFERENCES, KEY, CHAR, BYTE".replace(", ", ",").split(","))
     _TYPE_UNSIGNED, _TYPE_ZEROFILL = \
         map(CaselessKeyword, "UNSIGNED, ZEROFILL".replace(", ", ",").split(","))
+    _COL_ATTR_DISTKEY, _COL_ATTR_SORTKEY, _COL_ATTR_CHARACTER_SET = \
+        map(CaselessKeyword, "DISTKEY, SORTKEY, CHARACTER SET".replace(", ", ",").split(","))
+    _FK_MATCH = \
+        CaselessKeyword("MATCH") + Word(alphanums + "_")
+    _FK_ON, _FK_ON_OPT_RESTRICT, _FK_ON_OPT_CASCADE, _FK_ON_OPT_SET_NULL, _FK_ON_OPT_NO_ACTION = \
+        map(CaselessKeyword, "ON, RESTRICT, CASCADE, SET NULL, NO ACTION".replace(", ", ",").split(","))
+    _FK_ON_DELETE = \
+        _FK_ON + CaselessKeyword("DELETE") + (_FK_ON_OPT_RESTRICT | _FK_ON_OPT_CASCADE | _FK_ON_OPT_SET_NULL | _FK_ON_OPT_NO_ACTION)
+    _FK_ON_UPDATE = \
+        _FK_ON + CaselessKeyword("UPDATE") + (_FK_ON_OPT_RESTRICT | _FK_ON_OPT_CASCADE | _FK_ON_OPT_SET_NULL | _FK_ON_OPT_NO_ACTION)
     _SUPPRESS_QUOTE = _BACKQUOTE | _DOUBLEQUOTE
 
     _COMMENT = Suppress("--" + Regex(r".+"))
 
-    _COLUMN_CONSTRAINT_BASE = r"""
-    (?!--)
-    (
-        (
-            \s*\b(?:NOT\s+)NULL?\b
-        )?(
-            \s*\bAUTO_INCREMENT\b
-        )?(
-            \s*\b(UNIQUE|PRIMARY)(?:\s+KEY)?\b
-        )?(
-            \s*\bDEFAULT\b\s+(
-                ([A-Za-z0-9_\.\'\" -]|[^\x01-\x7E])*\:\:[A-Za-z0-9\[\]]+
-                |
-                \'(\\\'|[^\']|,)+\'
-                |
-                \"(\\\"|[^\"]|,)+\"
-                |
-                [^,]+
-            )
-        )?(
-            \s*\bCOMMENT\b\s+(
-                \'(\\\'|[^\']|,)+\'
-                |
-                \"(\\\"|[^\"]|,)+\"
-                |
-                [^,]+
-            )
-        )?(
-            \s*\bENCODE\s+\w+\b
-        )?(
-            \s*\b(?:NOT\s+)NULL?\b
-        )?(
-            \s*\b(UNIQUE|PRIMARY)(?:\s+KEY)?\b
-        )?
-    )
-    """
-
-    _COLUMN_CONSTRAINT = re.sub(r"(^\s+|\n)", r"", _COLUMN_CONSTRAINT_BASE, flags=re.MULTILINE)
-
 
     _CREATE_TABLE_STATEMENT = Suppress(_CREATE) + Optional(_TEMP)("temp") + Suppress(_TABLE) + Optional(Suppress(CaselessKeyword("IF NOT EXISTS"))) \
         + Optional(_SUPPRESS_QUOTE) + Optional(Word(alphanums + "_")("schema") + Optional(_SUPPRESS_QUOTE) + _DOT + Optional(_SUPPRESS_QUOTE)) + Word(alphanums + "_<>")("table") + Optional(_SUPPRESS_QUOTE) \
@@ -565,13 +622,16 @@ class DdlParse(DdlParseBase):
                             + Optional(Suppress(_REFERENCES)
                                 + Optional(_SUPPRESS_QUOTE) + Word(alphanums + "_")("references_table") + Optional(_SUPPRESS_QUOTE)
                                 + _LPAR + Group(delimitedList(Optional(_SUPPRESS_QUOTE) + Word(alphanums + "_") + Optional(_SUPPRESS_QUOTE)))("references_columns") + _RPAR
+                                + Optional(_FK_MATCH)("references_fk_match")  # MySQL
+                                + Optional(_FK_ON_DELETE)("references_fk_on_delete")  # MySQL
+                                + Optional(_FK_ON_UPDATE)("references_fk_on_update")  # MySQL
                             )
                         )
                     )
                 )("constraint")
                 |
                 Group(
-                    Optional(_SUPPRESS_QUOTE) + Word(alphanums + "_")("name") + Optional(_SUPPRESS_QUOTE)
+                    ((_SUPPRESS_QUOTE + Word(alphanums + " _")("name") + _SUPPRESS_QUOTE) ^ (Optional(_SUPPRESS_QUOTE) + Word(alphanums + "_")("name") + Optional(_SUPPRESS_QUOTE)))
                     + Group(
                         Group(
                             Word(alphanums + "_")
@@ -582,7 +642,22 @@ class DdlParse(DdlParseBase):
                         + Optional(_TYPE_ZEROFILL)("zerofill")
                     )("type")
                     + Optional(Word(r"\[\]"))("array_brackets")
-                    + Optional(Regex(_COLUMN_CONSTRAINT, re.IGNORECASE))("constraint")
+                    + Optional(
+                        Regex(r"(?!--)", re.IGNORECASE)
+                        + Group(
+                            Optional(Regex(r"\b(?:NOT\s+)NULL?\b", re.IGNORECASE))("null")
+                            & Optional(Regex(r"\bAUTO_INCREMENT\b", re.IGNORECASE))("auto_increment")
+                            & Optional(Regex(r"\b(UNIQUE|PRIMARY)(?:\s+KEY)?\b", re.IGNORECASE))("key")
+                            & Optional(Regex(
+                                r"\bDEFAULT\b\s+(?:((?:[A-Za-z0-9_\.\'\" -\{\}]|[^\x01-\x7E])*\:\:(?:character varying)?[A-Za-z0-9\[\]]+)|(?:\')((?:\\\'|[^\']|,)+)(?:\')|(?:\")((?:\\\"|[^\"]|,)+)(?:\")|([^,\s]+))",
+                                re.IGNORECASE))("default")
+                            & Optional(Regex(r"\bCOMMENT\b\s+(\'(\\\'|[^\']|,)+\'|\"(\\\"|[^\"]|,)+\"|[^,\s]+)", re.IGNORECASE))("comment")
+                            & Optional(Regex(r"\bENCODE\s+[A-Za-z0-9]+\b", re.IGNORECASE))("encode")  # Redshift
+                            & Optional(_COL_ATTR_DISTKEY)("distkey")  # Redshift
+                            & Optional(_COL_ATTR_SORTKEY)("sortkey")  # Redshift
+                            & Optional(Suppress(_COL_ATTR_CHARACTER_SET) + Word(alphanums + "_")("character_set"))  # MySQL
+                        )("constraint")
+                    )
                 )("column")
                 |
                 _COMMENT
